/* Sweetheart for PalmPilot application
   (Display an Image in 2 bit greyscale mode)
   (C) V1.0 Andreas Linke 1997 */

#pragma pack(2)
   
#include <Common.h>
#include <System/SysAll.h>
#include <UI/UIAll.h>

#include "greyscale.h" // greyscale mode switching routines
   
#define AppID 'Shrt'
#define DBType 'Data'
#define DBString "SweetheartDB"
   
#define SCREENSIZE (160*20*2) // 160 lines a 20*8 pixel * 2 bit

// open database
// create one if it doesn't exist
static DmOpenRef AppInit()
{
DmOpenRef db;
LocalID dbID;
Err err;
UInt mode=dmModeReadWrite;
UInt dbAttrs;

db=DmOpenDatabaseByTypeCreator(DBType,AppID,mode);
if(!db) /* data base not found, create one */
	{
	err=DmCreateDatabase(0,DBString,AppID,DBType,false);
	if(err==dmErrAlreadyExists)
		ErrFatalDisplayIf(err, "dmErrAlreadyExists");
	if(err==dmErrInvalidDatabaseName)
		ErrFatalDisplayIf(err, "dmErrInvalidDatabaseName");
	ErrFatalDisplayIf(err, "could not create Database");
	db=DmOpenDatabaseByTypeCreator(DBType,AppID,dmModeReadWrite);
}
err=DmOpenDatabaseInfo(db,&dbID, NULL, NULL, NULL, NULL);
ErrFatalDisplayIf(err, "Could not get database info.");
err = DmDatabaseInfo(0, dbID, NULL, &dbAttrs, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
ErrFatalDisplayIf(err, "Could not get database info.");
/* set the backup flag */
dbAttrs|=dmHdrAttrBackup;
err=DmSetDatabaseInfo(0,dbID,NULL,&dbAttrs,NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
ErrFatalDisplayIf(err, "Could not set database info.");

return db;
}


// terribly inefficient grey test routine
// uses DmWrite to write longs into a record
// only used for creating a SweetheartDB.pdb file
static void drawgreyscreen(VoidPtr pixels)
{
ULong ind=0;
int ix,iy,ib;
if(!pixels)
	return;
for(iy=0;iy<160;iy++)
	for(ix=0;ix<160/4/sizeof(ULong);ix++,ind+=sizeof(ULong))
		{
		ULong dat;
		for(dat=0,ib=0;ib<4*sizeof(ULong);ib++) // 4 pixel per byte
			{
			dat<<=2;
			if(iy<40)
				dat|= 0;
			else if(iy<80)
				dat|=1;
			else if(iy<120)
				dat|=2;
			else 
				dat|=3;
			}
		DmWrite(pixels,ind,&dat,sizeof(ULong));
		}
}
// get record from db
// create one and fill with test pattern if not exist
// lock
static VoidHand getrecord(DmOpenRef db)
{
VoidHand rh;
UInt ind=0;
/* create new record if no record exists */
rh=DmQueryRecord(db,ind);
if(!rh)
	{
	Ptr p;
	rh=DmNewRecord(db,&ind,SCREENSIZE);
	ErrFatalDisplayIf(!rh, "Could not create record");
	p=MemHandleLock(rh);
	drawgreyscreen(p);
	MemPtrUnlock(p);
	}
else rh=DmGetRecord(db,ind);
return rh;
}

DWord PilotMain(Word cmd, Ptr cmdPBP, Word launchFlags) 
{
EventType e;
short err;
unsigned char *vidptr;
DmOpenRef db;
VoidHand rh;

if (cmd != 0)
  return 0;

db=AppInit();
rh=getrecord(db);
// get a real pointer to memory and lock chunk in memory
vidptr=(unsigned char*)MemHandleLock(rh);

GraySwitchDisplayMode(1,&vidptr);
while(1)
  {
  EvtGetEvent(&e,-1);
  if(e.eType!=nilEvent)
  	{
  	// switch to b/w on every event (e.g. switch to launcher,alarm etc.)
  	GraySwitchDisplayMode(0,&vidptr);
	if(!SysHandleEvent(&e)) 
		if(!MenuHandleEvent((void *)0, &e, &err))
			/* do nothing */;
    if(e.eType==appStopEvent)
		break;
	// switch back to gray
	GraySwitchDisplayMode(1,&vidptr);
  	}
  }
/* leaving application */
MemPtrUnlock((Ptr)vidptr);
DmReleaseRecord(db,0,true); // release record and mark as dirty (will update on hotsync)
return 0;
}
